@@ -67,6 +67,10 @@ static int sndfd = -1;
 static int settings;
 static int arts_init_done = 0;
 static arts_stream_t stream = 0;
+static arts_stream_t record_stream = 0;
+static int bits = 0;
+static int speed = 0;
+static int channels = 0;
 static int frags;
 
 #if defined(HAVE_IOCTL_INT_INT_DOTS)
@@ -94,6 +98,7 @@ typedef int (*orig_open_ptr)(const char *pathname, int flags, ...);
 typedef int (*orig_close_ptr)(int fd);
 typedef int (*orig_ioctl_ptr)(int fd, ioctl_request_t request, ...);
 typedef ssize_t (*orig_write_ptr)(int fd, const void *buf, size_t count);
+typedef ssize_t (*orig_read_ptr)(int fd, void *buf, size_t count);
 typedef void* (*orig_mmap_ptr)(void *start, size_t length, int prot,
                                  int flags, int fd, off_t offset);
 typedef int (*orig_munmap_ptr)(void *start, size_t length);
@@ -104,6 +109,7 @@ static orig_open_ptr orig_open;
 static orig_close_ptr orig_close;
 static orig_ioctl_ptr orig_ioctl;
 static orig_write_ptr orig_write;
+static orig_read_ptr orig_read;
 static orig_mmap_ptr orig_mmap;
 static orig_munmap_ptr orig_munmap;
 static orig_fopen_ptr orig_fopen;
@@ -142,6 +148,7 @@ static void artsdsp_doinit()
 	orig_open = (orig_open_ptr)dlsym(RTLD_NEXT,"open");
 	orig_close = (orig_close_ptr)dlsym(RTLD_NEXT,"close");
 	orig_write = (orig_write_ptr)dlsym(RTLD_NEXT,"write");
+	orig_read = (orig_read_ptr)dlsym(RTLD_NEXT,"read");
 	orig_ioctl = (orig_ioctl_ptr)dlsym(RTLD_NEXT,"ioctl");
 	orig_mmap = (orig_mmap_ptr)dlsym(RTLD_NEXT,"mmap");
 	orig_munmap = (orig_munmap_ptr)dlsym(RTLD_NEXT,"munmap");
@@ -149,7 +156,11 @@ static void artsdsp_doinit()
 	orig_access = (orig_access_ptr)dlsym(RTLD_NEXT,"access");
 }
 
-static void artsdspdebug(const char *fmt,...)
+static void 
+#ifdef __GNUC__
+  __attribute__( ( format (  printf, 1, 2 ) ) )
+#endif
+artsdspdebug(const char *fmt,...)
 {
 	CHECK_INIT();
 
@@ -224,6 +235,7 @@ int open (const char *pathname, int flags, ...)
   settings = 0;
   frags = 0;
   stream = 0;
+  record_stream = 0;
 
   artsdspdebug ("aRts: hijacking /dev/dsp open...\n");
 
@@ -248,9 +260,6 @@ int open (const char *pathname, int flags, ...)
 
 int ioctl (int fd, ioctl_request_t request, ...)
 {
-  static int channels;
-  static int bits;
-  static int speed;
   int space, size, latency, odelay;
 
   /*
@@ -273,7 +282,7 @@ int ioctl (int fd, ioctl_request_t request, ...)
     {
       int *arg = (int *) argp;
       artsdspdebug("aRts: hijacking /dev/dsp ioctl (%d : %x - %p)\n",
-              fd, request, argp);
+              ( int ) fd, ( int ) request, ( void* ) argp);
 
       switch (request)
         {
@@ -400,9 +409,9 @@ int ioctl (int fd, ioctl_request_t request, ...)
 #ifdef SNDCTL_DSP_GETCAPS
         case SNDCTL_DSP_GETCAPS:			/* _SIOR ('P',15, int) */
 		  if(mmapemu)
-			*arg = DSP_CAP_MMAP | DSP_CAP_TRIGGER | DSP_CAP_REALTIME;
+			*arg = DSP_CAP_MMAP | DSP_CAP_TRIGGER | DSP_CAP_REALTIME | DSP_CAP_DUPLEX;
 		  else
-            *arg = 0;
+			*arg = DSP_CAP_DUPLEX;
           break;
 #endif
 
@@ -414,7 +423,7 @@ int ioctl (int fd, ioctl_request_t request, ...)
 
 #ifdef SNDCTL_DSP_SETTRIGGER
 		case SNDCTL_DSP_SETTRIGGER:         /* _SIOW ('P',16, int) */
-		  artsdspdebug("aRts: SNDCTL_DSP_SETTRIGGER(%d) unsupported\n");
+		  artsdspdebug("aRts: SNDCTL_DSP_SETTRIGGER unsupported\n");
 		  break;
 #endif
 
@@ -532,6 +541,11 @@ int close(int fd)
       {
         arts_close_stream(stream);
         stream = 0;
+      }
+      if (record_stream)
+      {
+          arts_close_stream(record_stream);
+          record_stream = 0;
       }
 	  if(mmapemu && mmapemu_obuffer)
 	  {
@@ -580,6 +594,21 @@ ssize_t write (int fd, const void *buf, size_t count)
   return 0;
 }
 
+ssize_t read (int fd, void *buf, size_t count)
+{
+  CHECK_INIT();
+
+  if (fd != sndfd)
+    return orig_read(fd,buf,count);
+  else if (sndfd == -1)
+    return 0;
+
+  if (!record_stream)
+    record_stream = arts_record_stream(speed, bits, channels, "artsdsp");
+  artsdspdebug ( "aRts: /dev/dsp read...\n" );
+  return arts_read(record_stream, buf, count);
+}
+
 void *mmap(void  *start,  size_t length, int prot, int flags,
              int fd, off_t offset)
 {
