@@ -63,7 +63,12 @@ namespace Arts {
 
 class AudioIOALSA : public AudioIO, public IONotify  {
 protected:
-	struct pollfd m_audio_read_pfd, m_audio_write_pfd;
+	// List of file descriptors
+        struct poll_descriptors {
+            poll_descriptors() : nfds(0), pfds(0) {};
+            int nfds;
+            struct pollfd *pfds;
+        } audio_write_pds, audio_read_pds;
 
         snd_pcm_t *m_pcm_playback;
 	snd_pcm_t *m_pcm_capture;
@@ -71,10 +76,11 @@ protected:
 	int m_period_size, m_periods;
 
         void startIO();
-        static int poll2iomanager(int pollTypes);
-        static int iomanager2poll(int pollTypes);
 	int setPcmParams(snd_pcm_t *pcm);
-	void watchDescriptor(snd_pcm_t *pcm, struct pollfd *pfd);
+        static int poll2iomanager(int pollTypes);
+        static int iomanager2poll(int ioTypes);
+	void getDescriptors(snd_pcm_t *pcm, poll_descriptors *pds);
+	void watchDescriptors(poll_descriptors *pds);
 
         void notifyIO(int fd, int types);
 
@@ -237,6 +243,11 @@ void AudioIOALSA::close()
 		m_pcm_playback = NULL;
 	}
 	Dispatcher::the()->ioManager()->remove(this, IOType::all);
+
+	delete[] audio_read_pds.pfds;
+	delete[] audio_write_pds.pfds;
+        audio_read_pds.pfds = NULL; audio_write_pds.pfds = NULL;
+        audio_read_pds.nfds = 0; audio_write_pds.nfds = 0;
 }
 
 void AudioIOALSA::setParam(AudioParam p, int& value)
@@ -252,25 +263,15 @@ void AudioIOALSA::setParam(AudioParam p, int& value)
 
 int AudioIOALSA::getParam(AudioParam p)
 {
-	snd_pcm_status_t *status;
-	snd_pcm_status_alloca(&status);
-
 	switch(p) {
-	case canRead:
+
+        case canRead:
 		if (! m_pcm_capture) return -1;
-		if (snd_pcm_status(m_pcm_capture, status) < 0) {
-			arts_warning("Capture channel status error!");
-			return -1;
-		}
-		return snd_pcm_frames_to_bytes(m_pcm_capture, snd_pcm_status_get_avail(status));
+		return snd_pcm_frames_to_bytes(m_pcm_capture, snd_pcm_avail_update(m_pcm_capture));
 
 	case canWrite:
 		if (! m_pcm_playback) return -1;
-		if(snd_pcm_status(m_pcm_playback, status) < 0) {
-			arts_warning("Playback channel status error!");
-			return -1;
-		}
-		return snd_pcm_frames_to_bytes(m_pcm_playback, snd_pcm_status_get_avail(status));
+		return snd_pcm_frames_to_bytes(m_pcm_playback, snd_pcm_avail_update(m_pcm_playback));
 
 	case selectReadFD:
 		return -1;
@@ -292,7 +293,7 @@ int AudioIOALSA::getParam(AudioParam p)
 			// No ALSA drivers in use...
 			return 0;
 		}
-		return 11;
+		return 15;
 	}
 
 	default:
@@ -302,12 +303,14 @@ int AudioIOALSA::getParam(AudioParam p)
 
 void AudioIOALSA::startIO()
 {
-        /* watch PCM file descriptor(s) */
+        /* get & watch PCM file descriptor(s) */
 	if (m_pcm_playback) {
-		watchDescriptor(m_pcm_playback, &m_audio_write_pfd);
+                getDescriptors(m_pcm_playback, &audio_write_pds);
+		watchDescriptors(&audio_write_pds);
         }
 	if (m_pcm_capture) {
-		watchDescriptor(m_pcm_capture, &m_audio_read_pfd);
+                getDescriptors(m_pcm_capture, &audio_read_pds);
+		watchDescriptors(&audio_read_pds);
         }
 
 }
@@ -326,37 +329,38 @@ int AudioIOALSA::poll2iomanager(int pollTypes)
 	return types;
 }
 
-int AudioIOALSA::iomanager2poll(int pollTypes)
+int AudioIOALSA::iomanager2poll(int ioTypes)
 {
 	int types = 0;
 
-	if(pollTypes & IOType::read)
+	if(ioTypes & IOType::read)
 		types |= POLLIN;
-	if(pollTypes & IOType::write)
+	if(ioTypes & IOType::write)
 		types |= POLLOUT;
-	if(pollTypes & IOType::except)
+	if(ioTypes & IOType::except)
 		types |= POLLERR;
 
 	return types;
 }
 
-void AudioIOALSA::watchDescriptor(snd_pcm_t *pcm, struct pollfd *pfd)
+void AudioIOALSA::getDescriptors(snd_pcm_t *pcm, poll_descriptors *pds)
 {
-        if (snd_pcm_poll_descriptors_count(pcm) != 1) {
-		arts_fatal("Can't handle more than one poll descriptor\n");
-		return;
-        }
-        if (snd_pcm_poll_descriptors(pcm, pfd, 1) != 1) {
-		arts_fatal("Cannot get poll descriptor\n");
-		return;
+	pds->nfds = snd_pcm_poll_descriptors_count(pcm);
+	pds->pfds = new struct pollfd[pds->nfds];
+
+        if (snd_pcm_poll_descriptors(pcm, pds->pfds, pds->nfds) != pds->nfds) {
+		arts_info("Cannot get poll descriptor(s)\n");
 	}
 
-        // See though the crack-fumes from the ALSA-developers and try to
-        // figure out which way this handle is supposed to be polled.
-        int types = poll2iomanager(pfd->events);
-
-        Dispatcher::the()->ioManager()->watchFD(pfd->fd, types, this);
+}
 
+void AudioIOALSA::watchDescriptors(poll_descriptors *pds)
+{
+	for(int i=0; i<pds->nfds; i++) {
+	        // Check in which direction this handle is supposed to be watched
+		int types = poll2iomanager(pds->pfds[i].events);
+		Dispatcher::the()->ioManager()->watchFD(pds->pfds[i].fd, types, this);
+	}
 }
 
 int AudioIOALSA::xrun(snd_pcm_t *pcm)
@@ -406,7 +410,6 @@ int AudioIOALSA::read(void *buffer, int size)
 
 int AudioIOALSA::write(void *buffer, int size)
 {
-
         int frames = snd_pcm_bytes_to_frames(m_pcm_playback, size);
 	int length;
 	while ((length = snd_pcm_writei(m_pcm_playback, buffer, frames)) < 0) {
@@ -422,7 +425,7 @@ int AudioIOALSA::write(void *buffer, int size)
 		}
 	}
 
-	if (length == frames) // Yet another DMix work-around
+	if (length == frames) // Sometimes the fragments are "odd" in alsa
 		return size;
 	else
 	        return snd_pcm_frames_to_bytes(m_pcm_playback, length);
@@ -432,28 +435,44 @@ void AudioIOALSA::notifyIO(int fd, int type)
 {
         int todo = 0;
 
-	// Call pcm_poll_desciptors_revents to translate the event safely for us.
-	// This is necassary even if we know what the event is, because _revents
-	// might have side-effects (dmix have)
-	unsigned short revents;
-        if(fd == m_audio_write_pfd.fd) {
-		m_audio_write_pfd.revents = iomanager2poll(type);
-		snd_pcm_poll_descriptors_revents(m_pcm_playback, &m_audio_write_pfd, 1, &revents);
-		if (revents & POLLOUT)
- 			todo |= AudioSubSystem::ioWrite;
-
+        // Translate from iomanager-types to poll-types,
+        // inorder to fake a snd_pcm_poll_descriptors_revents call.
+	if(m_pcm_playback) {
+	    for(int i=0; i < audio_write_pds.nfds; i++) {
+	        if(fd == audio_write_pds.pfds[i].fd) {
+                    audio_write_pds.pfds[i].revents = iomanager2poll(type);
+                    todo |= AudioSubSystem::ioWrite;
+                }
+            }
+            if (todo & AudioSubSystem::ioWrite) {
+                unsigned short revents;
+                snd_pcm_poll_descriptors_revents(m_pcm_playback,
+                                                 audio_write_pds.pfds,
+                                                 audio_write_pds.nfds,
+                                                 &revents);
+                if (! (revents & POLLOUT)) todo &= ~AudioSubSystem::ioWrite;
+            }
 	}
-        if(fd == m_audio_read_pfd.fd) {
-		m_audio_read_pfd.revents = iomanager2poll(type);
-		snd_pcm_poll_descriptors_revents(m_pcm_capture, &m_audio_read_pfd, 1, &revents);
-		if (revents & POLLIN)
- 			todo |= AudioSubSystem::ioRead;
+	if(m_pcm_capture) {
+	    for(int i=0; i < audio_read_pds.nfds; i++) {
+	        if(fd == audio_read_pds.pfds[i].fd) {
+                    audio_read_pds.pfds[i].revents = iomanager2poll(type);
+                    todo |= AudioSubSystem::ioRead;
+                }
+            }
+            if (todo & AudioSubSystem::ioRead) {
+                unsigned short revents;
+                snd_pcm_poll_descriptors_revents(m_pcm_capture,
+                                                 audio_read_pds.pfds,
+                                                 audio_read_pds.nfds,
+                                                 &revents);
+                if (! (revents & POLLIN)) todo &= ~AudioSubSystem::ioRead;
+            }
 	}
 
         if (type & IOType::except) todo |= AudioSubSystem::ioExcept;
 
-        if (todo) AudioSubSystem::the()->handleIO(todo);
-
+        if (todo != 0) AudioSubSystem::the()->handleIO(todo);
 }
 
 int AudioIOALSA::setPcmParams(snd_pcm_t *pcm)
