@@ -19,6 +19,8 @@
 #include "gslfilter.h"
 
 #include "gslfft.h"
+#include "gslsignal.h"
+#include <string.h>
 
 
 /* --- common utilities --- */
@@ -52,10 +54,10 @@ band_filter_common (unsigned int iorder,
   GslComplex fpoly[2 + 1] = { { 0, }, { 0, }, { 1, 0 } };
   double alpha, norm;
   guint i;
-  
+
   epsilon = gsl_trans_zepsilon2ss (epsilon);
   alpha = cos ((s_freq + p_freq) * 0.5) / cos ((s_freq - p_freq) * 0.5);
-  
+
   fpoly[0] = gsl_complex (1, 0);
   fpoly[1] = gsl_complex (1, 0);
   for (i = 0; i < iorder2; i++)
@@ -64,16 +66,16 @@ band_filter_common (unsigned int iorder,
       fpoly[1] = gsl_complex_mul (fpoly[1], gsl_complex_sub (gsl_complex (1, 0), gsl_complex_reciprocal (poles[i])));
     }
   norm = gsl_complex_div (fpoly[1], fpoly[0]).re;
-  
+
   if ((iorder2 & 1) == 0)      /* norm is fluctuation minimum */
     norm *= sqrt (1.0 / (1.0 + epsilon * epsilon));
-  
+
   /* z nominator polynomial */
   poly[0] = gsl_complex (norm, 0);
   for (i = 0; i < iorder2; i++)
     {
       GslComplex t, alphac = gsl_complex (alpha, 0);
-      
+
       t = band_pass ? gsl_complex_inv (roots[i]) : roots[i];
       fpoly[1] = gsl_complex_sub (gsl_complex_div (alphac, t), alphac);
       fpoly[0] = gsl_complex_inv (gsl_complex_reciprocal (t));
@@ -81,13 +83,13 @@ band_filter_common (unsigned int iorder,
     }
   for (i = 0; i <= iorder; i++)
     a[i] = poly[i].re;
-  
+
   /* z denominator polynomial */
   poly[0] = gsl_complex (1, 0);
   for (i = 0; i < iorder2; i++)
     {
       GslComplex t, alphac = gsl_complex (alpha, 0);
-      
+
       t = band_pass ? gsl_complex_inv (poles[i]) : poles[i];
       fpoly[1] = gsl_complex_sub (gsl_complex_div (alphac, t), alphac);
       fpoly[0] = gsl_complex_inv (gsl_complex_reciprocal (t));
@@ -108,7 +110,7 @@ filter_rp_to_z (unsigned int iorder,
 {
   GslComplex *poly = g_newa (GslComplex, iorder + 1);
   guint i;
-  
+
   /* z nominator polynomial */
   poly[0] = gsl_complex (1, 0);
   for (i = 0; i < iorder; i++)
@@ -269,26 +271,26 @@ gsl_filter_tscheb2_rp (unsigned int iorder,
   epsilon = gsl_trans_zepsilon2ss (epsilon);
   tepsilon = epsilon * tschebyscheff_eval (iorder, kappa_r / kappa_c);
   alpha = asinh (tepsilon) / order;
-  
+
   /* construct poles polynomial from tschebyscheff polynomial */
   for (i = 1; i <= iorder; i++)
     {
       double t = (i << 1) + iorder - 1;
       double beta = t * beta_mul;
-      
+
       root.re = sinh (alpha) * cos (beta);
       root.im = cosh (alpha) * sin (beta);
       root = gsl_complex_div (gsl_complex (kappa_r, 0), root);
       root = gsl_trans_s2z (root);
       poles[i - 1] = root;
     }
-  
+
   /* construct roots polynomial from tschebyscheff polynomial */
   for (i = 1; i <= iorder; i++)
     {
       double t = (i << 1) - 1;
       GslComplex root = gsl_complex (0, cos (t * beta_mul));
-      
+
       if (fabs (root.im) > 1e-14)
 	{
 	  root = gsl_complex_div (gsl_complex (kappa_r, 0), root);
@@ -364,7 +366,7 @@ gsl_filter_butter_lp (unsigned int iorder,
   GslComplex *roots = g_newa (GslComplex, iorder);
   GslComplex *poles = g_newa (GslComplex, iorder);
   double norm;
-  
+
   g_return_if_fail (freq > 0 && freq < GSL_PI);
 
   gsl_filter_butter_rp (iorder, freq, epsilon, roots, poles);
@@ -441,7 +443,7 @@ gsl_filter_tscheb2_lp (unsigned int iorder,
 
   gsl_filter_tscheb2_rp (iorder, freq, steepness, epsilon, roots, poles);
   filter_rp_to_z (iorder, roots, poles, a, b);
-  
+
   /* scale maximum to 1.0 */
   norm = gsl_poly_eval (iorder, b, 1) / gsl_poly_eval (iorder, a, 1); /* H(z=0):=1, e^(j*omega) for omega=0 => e^0==1 */
   gsl_poly_scale (iorder, a, norm);
@@ -583,7 +585,7 @@ gsl_filter_tscheb1_bp (unsigned int iorder,
   g_return_if_fail (freq1 > 0);
   g_return_if_fail (freq1 < freq2);
   g_return_if_fail (freq2 < GSL_PI);
-  
+
   theta = 2. * atan2 (1., cotan ((freq2 - freq1) * 0.5));
 
   gsl_filter_tscheb1_rp (iorder2, theta, epsilon, roots, poles);
@@ -619,7 +621,7 @@ gsl_filter_tscheb2_bp (unsigned int iorder,
   g_return_if_fail (freq1 > 0);
   g_return_if_fail (freq1 < freq2);
   g_return_if_fail (freq2 < GSL_PI);
-  
+
   theta = 2. * atan2 (1., cotan ((freq2 - freq1) * 0.5));
 
   gsl_filter_tscheb2_rp (iorder2, theta, steepness, epsilon, roots, poles);
@@ -689,7 +691,7 @@ gsl_filter_tscheb1_bs (unsigned int iorder,
   g_return_if_fail (freq1 > 0);
   g_return_if_fail (freq1 < freq2);
   g_return_if_fail (freq2 < GSL_PI);
-  
+
   theta = 2. * atan2 (1., tan ((freq2 - freq1) * 0.5));
 
   gsl_filter_tscheb1_rp (iorder2, theta, epsilon, roots, poles);
@@ -725,7 +727,7 @@ gsl_filter_tscheb2_bs (unsigned int iorder,
   g_return_if_fail (freq1 > 0);
   g_return_if_fail (freq1 < freq2);
   g_return_if_fail (freq2 < GSL_PI);
-  
+
   theta = 2. * atan2 (1., tan ((freq2 - freq1) * 0.5));
 
   gsl_filter_tscheb2_rp (iorder2, theta, steepness, epsilon, roots, poles);
@@ -749,14 +751,14 @@ tschebyscheff_poly (unsigned int degree,
   else
     {
       double *u = g_newa (double, 1 + degree);
-      
+
       u[degree] = 0; u[degree - 1] = 0;
       tschebyscheff_poly (degree - 2, u);
-      
+
       v[0] = 0;
       tschebyscheff_poly (degree - 1, v + 1);
       gsl_poly_scale (degree - 1, v + 1, 2);
-      
+
       gsl_poly_sub (degree, v, u);
     }
 }
@@ -774,10 +776,10 @@ gsl_filter_tscheb1_test	(unsigned int iorder,
   double *q = g_newa (double, 2 * (1 + iorder));
   double O = gsl_trans_freq2s (zomega);
   unsigned int i;
-  
+
   /* calc Vk() */
   tschebyscheff_poly (iorder, vk);
-  
+
   /* calc q=1+e^2*Vk()^2 */
   gsl_poly_mul (q, iorder >> 1, vk, iorder >> 1, vk);
   iorder *= 2;
@@ -788,7 +790,7 @@ gsl_filter_tscheb1_test	(unsigned int iorder,
   gsl_poly_complex_roots (iorder, q, roots);
   for (i = 0; i < iorder; i++)
     roots[i] = gsl_complex_mul (roots[i], gsl_complex (0, O));
-  
+
   /* choose roots from the left half-plane */
   if (0)
     g_print ("zhqr-root:\n%s\n", gsl_complex_list (iorder, roots, "  "));
@@ -801,23 +803,23 @@ gsl_filter_tscheb1_test	(unsigned int iorder,
 	r++;
       }
   iorder /= 2;
-  
+
   /* assert roots found */
   if (!(r - roots == iorder))
     {
       g_print ("ERROR: n_roots=%u != iorder=%u\n", r - roots, iorder);
       abort ();
     }
-  
+
   /* s => z */
   for (i = 0; i < iorder; i++)
     roots[i] = gsl_trans_s2z (roots[i]);
-  
+
   /* z denominator polynomial */
   gsl_cpoly_from_roots (iorder, zf, roots);
   for (i = 0; i <= iorder; i++)
     b[i] = zf[i].re;
-  
+
   /* z nominator polynomial */
   for (i = 0; i < iorder; i++)
     {
@@ -827,7 +829,7 @@ gsl_filter_tscheb1_test	(unsigned int iorder,
   gsl_cpoly_from_roots (iorder, zf, roots);
   for (i = 0; i <= iorder; i++)
     a[i] = zf[i].re;
-  
+
   /* scale for b[0]==1.0 */
   gsl_poly_scale (iorder, b, 1.0 / b[0]);
 
@@ -855,7 +857,7 @@ gsl_blackman_window (double x)
 /**
  * gsl_filter_fir_approx
  *
- * @iorder: order of the filter (must be odd, >= 3)
+ * @iorder: order of the filter (must be oven, >= 2)
  * @freq:   the frequencies of the transfer function
  * @value:  the desired value of the transfer function
  *
@@ -881,22 +883,22 @@ gsl_filter_fir_approx (unsigned int  iorder,
   double lfreq = -2, lval = 1.0, rfreq = -1, rval = 1.0;
   double *fft_in, *fft_out;
   double ffact;
-  
+
   g_return_if_fail (iorder >= 2);
   g_return_if_fail ((iorder & 1) == 0);
 
   while (fft_size / 2 <= iorder)
     fft_size *= 2;
-  
+
   fft_in = g_newa (double, fft_size*2);
   fft_out = fft_in+fft_size;
   ffact = 2.0 * GSL_PI / (double)fft_size;
-  
+
   for (i = 0; i <= fft_size / 2; i++)
     {
       double f = (double) i * ffact;
       double pos, val;
-      
+
       while (f > rfreq && point != n_points)
 	{
 	  lfreq = rfreq;
@@ -905,10 +907,10 @@ gsl_filter_fir_approx (unsigned int  iorder,
 	  rval = value[point];
 	  point++;
 	}
-      
+
       pos = (f - lfreq) / (rfreq - lfreq);
       val = lval * (1.0 - pos) + rval * pos;
-      
+
       if (i != fft_size / 2)
 	{
 	  fft_in[2 * i] = val;
@@ -917,9 +919,9 @@ gsl_filter_fir_approx (unsigned int  iorder,
       else
 	fft_in[1] = val;
     }
-  
+
   gsl_power2_fftsr (fft_size, fft_in, fft_out);
-  
+
   for (i = 0; i <= iorder / 2; i++)
     {
       double c = fft_out[i] * gsl_blackman_window (0.5 + (double) i / (iorder + 2.0));
@@ -966,7 +968,7 @@ gsl_iir_filter_change (GslIIRFilter  *f,
 
   g_return_if_fail (f != NULL && a != NULL && b != NULL && buffer != NULL);
   g_return_if_fail (order > 0);
-  
+
   /* there's no point in calling this function if f wasn't setup properly
    * and it's only the As and Bs that changed
    */
@@ -1049,12 +1051,12 @@ filter_step_direct_canon_1 (GslIIRFilter *f,
 
 void
 gsl_iir_filter_eval (GslIIRFilter *f,
+		     guint         n_values,
 		     const gfloat *x,
-		     gfloat       *y,
-		     guint         n_values)
+		     gfloat       *y)
 {
   const gfloat *bound;
-  
+
   g_return_if_fail (f != NULL && x != NULL && y != NULL);
   g_return_if_fail (f->order > 0);
 
@@ -1068,9 +1070,263 @@ gsl_iir_filter_eval (GslIIRFilter *f,
 }
 
 
+/* --- biquad filters --- */
+void
+gsl_biquad_config_init (GslBiquadConfig   *c,
+			GslBiquadType      type,
+			GslBiquadNormalize normalize)
+{
+  g_return_if_fail (c != NULL);
+
+  memset (c, 0, sizeof (*c));
+  c->type = type;
+  c->normalize = normalize;
+  gsl_biquad_config_setup (c, 0.5, 3, 1);
+  c->approx_values = TRUE;	/* need _setup() */
+}
+
+void
+gsl_biquad_config_setup (GslBiquadConfig *c,
+			 gfloat           f_fn,
+			 gfloat           gain,
+			 gfloat           quality)
+{
+  g_return_if_fail (c != NULL);
+  g_return_if_fail (f_fn >= 0 && f_fn <= 1);
+
+  if (c->type == GSL_BIQUAD_RESONANT_HIGHPASS)
+    f_fn = 1.0 - f_fn;
+  c->f_fn = f_fn;			/* nyquist relative (0=DC, 1=nyquist) */
+  c->gain = gain;
+  c->quality = quality;			/* FIXME */
+  c->k = tan (c->f_fn * GSL_PI / 2.);
+  c->v = pow (10, c->gain / 20.);	/* v=10^(gain[dB]/20) */
+  c->dirty = TRUE;
+  c->approx_values = FALSE;
+}
+
+void
+gsl_biquad_config_approx_freq (GslBiquadConfig *c,
+			       gfloat           f_fn)
+{
+  g_return_if_fail (f_fn >= 0 && f_fn <= 1);
+
+  if (c->type == GSL_BIQUAD_RESONANT_HIGHPASS)
+    f_fn = 1.0 - f_fn;
+  c->f_fn = f_fn;                       /* nyquist relative (0=DC, 1=nyquist) */
+  c->k = tan (c->f_fn * GSL_PI / 2.);	/* FIXME */
+  c->dirty = TRUE;
+  c->approx_values = TRUE;
+}
+
+void
+gsl_biquad_config_approx_gain (GslBiquadConfig *c,
+			       gfloat           gain)
+{
+  c->gain = gain;
+  c->v = gsl_approx_exp2 (c->gain * GSL_LOG2POW20_10);
+  c->dirty = TRUE;
+  c->approx_values = TRUE;
+}
+
+static void
+biquad_lpreso (GslBiquadConfig *c,
+	       GslBiquadFilter *f)
+{
+  gdouble kk, sqrt2_reso, denominator;
+  gdouble r2p_norm = 0;			/* resonance gain to peak gain (pole: -sqrt2_reso+-j) */
+
+  kk = c->k * c->k;
+  sqrt2_reso = 1 / c->v;
+  denominator = 1 + (c->k + sqrt2_reso) * c->k;
+
+  switch (c->normalize)
+    {
+    case GSL_BIQUAD_NORMALIZE_PASSBAND:
+      r2p_norm = kk;
+      break;
+    case GSL_BIQUAD_NORMALIZE_RESONANCE_GAIN:
+      r2p_norm = kk * sqrt2_reso;
+      break;
+    case GSL_BIQUAD_NORMALIZE_PEAK_GAIN:
+      r2p_norm = (GSL_SQRT2 * sqrt2_reso - 1.0) / (sqrt2_reso * sqrt2_reso - 0.5);
+      r2p_norm = r2p_norm > 1 ? kk * sqrt2_reso : kk * r2p_norm * sqrt2_reso;
+      break;
+    }
+  f->xc0 = r2p_norm / denominator;
+  f->xc1 = 2 * f->xc0;
+  f->xc2 = f->xc0;
+  f->yc1 = 2 * (kk - 1) / denominator;
+  f->yc2 = (1 + (c->k - sqrt2_reso) * c->k) / denominator;
+}
+
+void
+gsl_biquad_filter_config (GslBiquadFilter *f,
+			  GslBiquadConfig *c,
+			  gboolean         reset_state)
+{
+  g_return_if_fail (f != NULL);
+  g_return_if_fail (c != NULL);
+
+  if (c->dirty)
+    {
+      switch (c->type)
+	{
+	case GSL_BIQUAD_RESONANT_LOWPASS:
+	  biquad_lpreso (c, f);
+	  break;
+	case GSL_BIQUAD_RESONANT_HIGHPASS:
+	  biquad_lpreso (c, f);
+	  f->xc1 = -f->xc1;
+	  f->yc1 = -f->yc1;
+	  break;
+	default:
+	  g_assert_not_reached ();
+	}
+      c->dirty = FALSE;
+    }
+
+  if (reset_state)
+    f->xd1 = f->xd2 = f->yd1 = f->yd2 = 0;
+}
+
+void
+gsl_biquad_filter_eval (GslBiquadFilter *f,
+			guint            n_values,
+			const gfloat    *x,
+			gfloat          *y)
+{
+  const gfloat *bound;
+  gdouble xc0, xc1, xc2, yc1, yc2, xd1, xd2, yd1, yd2;
+
+  g_return_if_fail (f != NULL && x != NULL && y != NULL);
+
+  xc0 = f->xc0;
+  xc1 = f->xc1;
+  xc2 = f->xc2;
+  yc1 = f->yc1;
+  yc2 = f->yc2;
+  xd1 = f->xd1;
+  xd2 = f->xd2;
+  yd1 = f->yd1;
+  yd2 = f->yd2;
+  bound = x + n_values;
+  while (x < bound)
+    {
+      gdouble k0, k1, k2;
+
+      k2 = xd2 * xc2;
+      k1 = xd1 * xc1;
+      xd2 = xd1;
+      xd1 = *x++;
+      k2 -= yd2 * yc2;
+      k1 -= yd1 * yc1;
+      yd2 = yd1;
+      k0 = xd1 * xc0;
+      yd1 = k2 + k1;
+      *y++ = yd1 += k0;
+    }
+  f->xd1 = xd1;
+  f->xd2 = xd2;
+  f->yd1 = yd1;
+  f->yd2 = yd2;
+}
+
+#if 0
+void
+gsl_biquad_lphp_reso (GslBiquadFilter   *c,
+		      gfloat             f_fn,	/* nyquist relative (0=DC, 1=nyquist) */
+		      float              gain,
+		      gboolean		 design_highpass,
+		      GslBiquadNormalize normalize)
+{
+  double k, kk, v;
+  double sqrt2_reso;
+  double denominator;
+  double r2p_norm = 0;			/* resonance gain to peak gain (pole: -sqrt2_reso+-j) */
+
+  g_return_if_fail (c != NULL);
+  g_return_if_fail (f_fn >= 0 && f_fn <= 1);
+
+  if (design_highpass)
+    f_fn = 1.0 - f_fn;
+
+  v = pow (10, gain / 20.);		/* v=10^(gain[dB]/20) */
+  k = tan (f_fn * GSL_PI / 2.);
+  kk = k * k;
+  sqrt2_reso = 1 / v;
+  denominator = 1 + (k + sqrt2_reso) * k;
+
+  if (0)
+    g_printerr ("BIQUAD-lp: R=%f\n", GSL_SQRT2 * sqrt2_reso);
+
+  switch (normalize)
+    {
+    case GSL_BIQUAD_NORMALIZE_PASSBAND:
+      r2p_norm = kk;
+      break;
+    case GSL_BIQUAD_NORMALIZE_RESONANCE_GAIN:
+      r2p_norm = kk * sqrt2_reso;
+      break;
+    case GSL_BIQUAD_NORMALIZE_PEAK_GAIN:
+      r2p_norm = (GSL_SQRT2 * sqrt2_reso - 1.0) / (sqrt2_reso * sqrt2_reso - 0.5);
+      g_print ("BIQUAD-lp: (peak-gain) r2p_norm = %f \n", r2p_norm);
+      r2p_norm = r2p_norm > 1 ? kk * sqrt2_reso : kk * r2p_norm * sqrt2_reso;
+      break;
+    }
+  c->xc0 = r2p_norm / denominator;
+  c->xc1 = 2 * c->xc0;
+  c->xc2 = c->xc0;
+  c->yc1 = 2 * (kk - 1) / denominator;
+  c->yc2 = (1 + (k - sqrt2_reso) * k) / denominator;
+
+  if (design_highpass)
+    {
+      c->xc1 = -c->xc1;
+      c->yc1 = -c->yc1;
+    }
+  /* normalization notes:
+   * pole: -sqrt2_reso+-j
+   * freq=0.5: reso->peak gain=8adjust:0.9799887, 9adjust:0.98415
+   * resonance gain = 1/(1-R)=sqrt2_reso
+   * sqrt2_reso*(1-R)=1
+   * 1-R=1/sqrt2_reso
+   * R= 1-1/sqrt2_reso
+   * peak gain = 2/(1-R^2)
+   * = 2 * (1 - (1 - 1 / sqrt2_reso) * (1 - 1 / sqrt2_reso))
+   * = 2 - 2 * (1 - 1 / sqrt2_reso)^2
+   */
+}
+#endif
+
+
 /* --- filter scanning -- */
 #define SINE_SCAN_SIZE 1024
 
+/**
+ * gsl_filter_sine_scan
+ *
+ * @order:    order of the iir filter
+ * @a:        root polynomial coefficients of the filter a[0..order]
+ * @b:        pole polynomial coefficients of the filter b[0..order]
+ * @freq:     frequency to test
+ * @n_values: number of samples
+ *
+ * This function sends a sine signal of the desired frequency through an IIR
+ * filter, to test the value of the transfer function at a given point. It uses
+ * gsl_iir_filter_eval to do so.
+ *
+ * Compared to a "mathematical approach" of finding the transfer function,
+ * this function makes it possible to see the effects of finite arithmetic
+ * during filter evaluation.
+ *
+ * The first half of the output signal is not considered, since a lot of IIR
+ * filters have a transient phase where also overshoot is possible.
+ *
+ * For n_values, you should specify a reasonable large value. It should be
+ * a lot larger than the filter order, and large enough to let the input
+ * signal become (close to) 1.0 multiple times.
+ */
 gdouble
 gsl_filter_sine_scan (guint order,
                       const gdouble *a,
@@ -1084,33 +1340,33 @@ gsl_filter_sine_scan (guint order,
   GslIIRFilter filter;
   gdouble *filter_state;
   guint scan_start = n_values / 2;
-  
+
   g_return_val_if_fail (order > 0, 0.0);
   g_return_val_if_fail (a != NULL, 0.0);
   g_return_val_if_fail (b != NULL, 0.0);
   g_return_val_if_fail (freq > 0 && freq < GSL_PI, 0.0);
   g_return_val_if_fail (n_values > 0, 0.0);
-  
+
   filter_state = g_newa (double, (order + 1) * 4);
   gsl_iir_filter_setup (&filter, order, a, b, filter_state);
-  
+
   while (n_values)
     {
       guint todo = MIN (n_values, SINE_SCAN_SIZE);
       guint i;
-      
+
       for (i = 0; i < todo; i++)
 	{
 	  x[i] = sin (pos);
 	  pos += freq;
 	}
-      
-      gsl_iir_filter_eval (&filter, x, y, SINE_SCAN_SIZE);
-      
+
+      gsl_iir_filter_eval (&filter, SINE_SCAN_SIZE, x, y);
+
       for (i = 0; i < todo; i++)
         if (n_values - i < scan_start)
 	  result = MAX (y[i], result);
-      
+
       n_values -= todo;
     }
   return result;
