@@ -173,7 +173,7 @@ typedef enum
   MAGIC_CHECK_UINT_ONES,
   MAGIC_CHECK_STRING_EQUAL,
   MAGIC_CHECK_STRING_GREATER,
-  MAGIC_CHECK_STRING_SMALLER,
+  MAGIC_CHECK_STRING_SMALLER
 } MagicCheckType;
 typedef union
 {
