@@ -840,10 +840,15 @@ StdFlowSystem::StdFlowSystem()
 	static bool gsl_is_initialized = false;
 	if(!gsl_is_initialized)
 	{
-		GslConfigValue values[2] = { { "wave_chunk_padding", 8 }, { 0, 0 } };
+		GslConfigValue values[3] = {
+			{ "wave_chunk_padding", 8 },
+			{ "dcache_block_size",  4000, },
+			{ 0, 0 }
+		};
 		gsl_is_initialized = true;
 
-                if (!g_thread_supported ()) g_thread_init(0);
+		if (!g_thread_supported ())
+			g_thread_init(0);
 		gsl_init(values, (GslMutexTable *)gslGlobalMutexTable);
 
 		/*
