@@ -212,7 +212,7 @@ Dispatcher::Dispatcher(IOManager *ioManager, StartServer startServer)
 	 * setup signal handler for SIGPIPE
 	 */
 	orig_sigpipe = signal(SIGPIPE,SIG_IGN);
-	if(orig_sigpipe != SIG_DFL)
+	if((orig_sigpipe != SIG_DFL) && (orig_sigpipe != SIG_IGN))
 	{
 		cerr << "mcop warning: user defined signal handler found for"
 		        " SIG_PIPE, overriding" << endl;
