@@ -28,11 +28,10 @@
 #ifdef HAVE_LIBPTHREAD
 
 #include <gsl/gslconfig.h>
-#if (GSL_HAVE_MUTEXATTR_SETTYPE > 0)
-/* use full pthread facilities (unix98) if the configure check succeeded*/
-#define	_XOPEN_SOURCE	500
-#endif
 
+#include <sys/types.h>
+#include <stddef.h>
+#include <stdarg.h>
 #include <pthread.h>
 #include <semaphore.h>
 #include <debug.h>
