@@ -2,6 +2,7 @@
 
     Copyright (C) 2000-2001 Stefan Westerfeld
                             stefan@space.twc.de
+                       2003 Arnold Krille <arnold@arnoldarts.de
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     Permission is also granted to link this program with the Qt
     library, treating Qt like a library that normally accompanies the
@@ -35,6 +36,7 @@
 #include "cpuusage.h"
 #include "debug.h"
 #include "artsversion.h"
+#include "crashhandler.h"
 
 using namespace std;
 using namespace Arts;
@@ -67,7 +69,7 @@ static void exitUsage(const char *progname)
 	fprintf(stderr,"-r <samplingrate>   set samplingrate to use\n");
 	fprintf(stderr,"-b <bits>           set number of bits (8 or 16)\n");
 	fprintf(stderr,"-d                  enable full duplex operation\n");
-	fprintf(stderr,"-V <volume>         set output volume\n");
+	fprintf(stderr,"-V <volume>[dB]     set output volume\n");
 	fprintf(stderr,"-D <devicename>     audio device (usually /dev/dsp)\n");
 	fprintf(stderr,"-F <fragments>      number of fragments\n");
 	fprintf(stderr,"-S <size>           fragment size in bytes\n");
@@ -81,6 +83,7 @@ static void exitUsage(const char *progname)
 	fprintf(stderr,"-l <level>          information level\n");
 	fprintf(stderr,"  3: quiet, 2: warnings, 1: info, 0: debug\n");
 	fprintf(stderr,"-m <appName>        application to display messages\n");
+	fprintf(stderr,"-c <appName>        application to display crash dialogs\n");
 	exit(1);	
 }
 
@@ -107,6 +110,7 @@ static int  					cfgFragmentSize	= 0;
 static int  					cfgPort			= 0;
 static int  					cfgDebugLevel	= 2;
 static const char			   *cfgDebugApp		= 0;
+static const char			   *cfgCrashApp		= 0;
 static bool  					cfgFullDuplex	= 0;
 static bool  					cfgForceStart	= 0;
 static const char			   *cfgDeviceName   = 0;
@@ -118,14 +122,14 @@ static const char              *cfgAudioIO      = 0;
 #endif
 static int                      cfgAutoSuspend  = 0;
 static int                      cfgBuffers      = 0;
-static float			cfgVolume = 0.0;
+static float					cfgVolume		= 0.0;
 
 static bool						cmdListAudioIO  = false;
 
 static void handleArgs(int argc, char **argv)
 {
 	int optch;
-	while((optch = getopt(argc,argv,"r:p:nuF:S:hD:dl:a:Ab:s:m:vNw:fV:")) > 0)
+	while((optch = getopt(argc,argv,"r:p:nuF:S:hD:dl:a:Ab:s:m:vNw:fV:c:")) > 0)
 	{
 		switch(optch)
 		{
@@ -152,6 +156,8 @@ static void handleArgs(int argc, char **argv)
 				break;
 			case 'm': cfgDebugApp = optarg;
 				break;
+			case 'c': cfgCrashApp = optarg;
+				break;
 			case 'u': cfgServers = static_cast<Dispatcher::StartServer>( cfgServers | Dispatcher::noAuthentication);
 				break;
 			case 'A': cmdListAudioIO = true;
@@ -165,7 +171,13 @@ static void handleArgs(int argc, char **argv)
 				break;
 			case 'f': cfgForceStart = true;
 				break;
-			case 'V': cfgVolume = atof(optarg);
+			case 'V':
+				if ( strstr( optarg,"dB" )&& strlen( strstr( optarg,"dB" ) )==2 ) {
+					char* val = ( char* )calloc( strlen( optarg )-1, sizeof( char ) );
+					strncpy( val, optarg, strlen( optarg )-2 );
+					cfgVolume = pow( 10, atof( val )/20.0 );
+					free( val );
+				} else cfgVolume = atof( optarg );
 				break;
 			case 'h':
 			default:
@@ -246,11 +258,22 @@ int main(int argc, char **argv)
 	arts_debug("artsd version is %s",ARTS_VERSION);
 	if (cfgDebugApp)
 		Debug::messageApp(cfgDebugApp);
+	if (cfgCrashApp)
+	{
+		CrashHandler::setCrashHandler(CrashHandler::defaultCrashHandler);
+		CrashHandler::setCrashApp(cfgCrashApp);
+		CrashHandler::setApplicationName("artsd");
+		CrashHandler::setApplicationPath(argc ? argv[0] : "artsd");
+  		CrashHandler::setApplicationVersion (ARTS_VERSION);
+		CrashHandler::setProgramName("Soundserver");
+  		CrashHandler::setBugAddress("submit@bugs.kde.org");
+	}
 
 	if(cfgPort)			 TCPServer::setPort(cfgPort);
 
 	CPUUsage	cpuUsage;
 	Dispatcher	dispatcher(0,cfgServers);
+	string      warnNullDevice;
 
 	initSignals();
 
@@ -266,16 +289,15 @@ int main(int argc, char **argv)
 	if(cfgDeviceName)	 AudioSubSystem::the()->deviceName(cfgDeviceName);
 	if(cfgBits)			 AudioSubSystem::the()->format(cfgBits);
 
+	SoundServerStartup startup;
+	startup.lock();
+
 	if(cfgForceStart && !AudioSubSystem::the()->check())
 	{
-		/* hack to get this message to the user in any case */
-		Debug::init("[artsd]", static_cast<Debug::Level>(1));
-		arts_info(
-			"Error while initializing the sound driver:\n"
-			"%s\n\n"
-			"The sound server will continue, using the null output device.",
-			AudioSubSystem::the()->error());
-		Debug::init("[artsd]", static_cast<Debug::Level>(cfgDebugLevel));
+		warnNullDevice  = "Error while initializing the sound driver:\n";
+		warnNullDevice += AudioSubSystem::the()->error();
+		warnNullDevice += "\n\nThe sound server will continue, using the null output device.";
+		
 		AudioSubSystem::the()->audioIO("null");
 	}
 
@@ -307,6 +329,17 @@ int main(int argc, char **argv)
 		if(!publishReferences(server,audioManager,false)) return 1;
 	}
 
+	startup.unlock();
+
+	/* warn if we are using the null device */
+	if (warnNullDevice != "")
+	{
+		/* hack to get this message to the user in any case */
+		Debug::init("[artsd]", static_cast<Debug::Level>(1));
+		arts_info("%s", warnNullDevice.c_str());
+		Debug::init("[artsd]", static_cast<Debug::Level>(cfgDebugLevel));
+	}
+
 	/* warn if there was a problem with artswrapper */
 	char *wrapper = getenv("STARTED_THROUGH_ARTSWRAPPER");
 	if (wrapper && !strcmp(wrapper, "2"))
@@ -336,4 +369,6 @@ REGISTER_IMPLEMENTATION(SimpleSoundServer_impl);
 REGISTER_IMPLEMENTATION(SoundServer_impl);
 #include "soundserverv2_impl.h"
 REGISTER_IMPLEMENTATION(SoundServerV2_impl);
+#include "soundserverstartup_impl.h"
+REGISTER_IMPLEMENTATION(SoundServerStartup_impl);
 #endif
