@@ -4,6 +4,8 @@
                             stefan@space.twc.de
                        2001 Matthias Kretz
                             kretz@kde.org
+                       2003 Allan Sandfeld Jensen
+                            kde@carewolf.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,7 +19,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     Permission is also granted to link this program with the Qt
     library, treating Qt like a library that normally accompanies the
@@ -59,6 +61,19 @@ public:
 	bool done();
 };
 
+class PlayObjectJob : public SoundServerJob
+{
+protected:
+	PlayObject plob;
+	bool terminated;
+
+public:
+	PlayObjectJob(PlayObject newPlob);
+
+	void terminate();
+	bool done();
+};
+
 class PlayStreamJob : public SoundServerJob
 {
 protected:
