@@ -61,14 +61,14 @@ public:
 		float timesec = flpos / (float)wav->samplingRate;
 		float timems = (timesec - floor(timesec)) * 1000.0;
 
-		return poTime(timesec, timems, (int)flpos, "samples");
+		return poTime( int( timesec ), int( timems ), (int)flpos, "samples" );
 	}
 
 	poTime overallTime() {
 		float timesec = (float)sampleCount() / (float)wav->samplingRate;
 		float timems = (timesec - floor(timesec)) * 1000.0;
 
-		return poTime(timesec, timems, sampleCount(), "samples");
+		return poTime( int( timesec ), int( timems ), sampleCount(), "samples" );
 	}
 
 	poCapabilities capabilities() {
